#' Reclassify landscape rasters by SDM
#'
#' Prepare for fitting SDMs by reclassifying a landscape rasters according to
#' the classifications used by a specific species distribution model (SDM).
#'
#' This function is called by [python_focal_prep()] on a set of landscape
#' rasters generated by segregating an input landscape by class. It is not
#' intended to be called directly.
#'
#' @param landscape_stack SpatRaster created by [terra::rast()]
#' @param SDM The name of intended species distribution model: `"riparian"`,
#'   `"waterbird_fall"`, or `"waterbird_win"`
#'
#' @return SpatRaster with layers for each land cover class
#' @importFrom stats setNames
#' @export
#'
#' @examples
#' #r <- terra::rast(matrix(sample(c(0:4), size = 100, replace = TRUE),
#' #         ncol = 10, nrow = 10)
#' #levels(r) <- c('RIPARIAN_FOREST_POFR', 'RIPARIAN_FOREST_QULO', 'WETLAND', 'ORCHARD', 'VINEYARD')
#' #layernames = terra::freq(r)$label
#' #s = terra::segregate(r, other = 0)
#' #names(s) = terra::freq(r)$label
#' #example = reclassify_landcover(s, SDM = 'RIPARIAN')

reclassify_landcover = function(landscape_stack, SDM) {
  if (SDM == 'riparian') {
    res = c(
      # sum of all riparian subclasses
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('RIPARIAN', names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('RIPARIAN'),
      # sum of all wetland subclasses
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('WETLAND', names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('WETLAND'),
      # sum of perennial crops, grassland/pasture, and other ag
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('ORCH|VINE', names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('ORCHVIN'),
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('PASTURE|GRASS',
                                     names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('GRASSPAS'),
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('FIELD|GRAIN|ROW',
                                     names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('AG'),
      # sum of open water and submerged veg
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('WATER|_SUB',
                                     names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('WATER')
    )
    # rename riparian and wetland subclasses if present:
    if ('WETLAND_MANAGED_PERENNIAL' %in% names(landscape_stack)) {
      res = c(res,
              terra::subset(landscape_stack, 'WETLAND_MANAGED_PERENNIAL') %>%
                stats::setNames('PERM'))
    }
    if (any(grepl('FOREST_POFR', names(landscape_stack)))) {
      res = c(res,
              terra::subset(landscape_stack,
                            names(landscape_stack)[grepl('FOREST_POFR', names(landscape_stack))]) %>%
                sum(na.rm = TRUE) %>%
                stats::setNames('POFR'))
    }
    if (any(grepl('FOREST_QULO', names(landscape_stack)))) {
      res = c(res,
              terra::subset(landscape_stack,
                            names(landscape_stack)[grepl('FOREST_POFR', names(landscape_stack))]) %>%
                sum(na.rm = TRUE) %>%
                stats::setNames('QULO'))
    }
    if (any(grepl('FOREST_SALIX', names(landscape_stack)))) {
      res = c(res,
              terra::subset(landscape_stack,
                            names(landscape_stack)[grepl('FOREST_SALIX', names(landscape_stack))]) %>%
                sum(na.rm = TRUE) %>%
                stats::setNames('SALIX'))
    }
    if (any(grepl('FOREST_MIXED', names(landscape_stack)))) {
      res = c(res,
              terra::subset(landscape_stack,
                            names(landscape_stack)[grepl('FOREST_MIXED', names(landscape_stack))]) %>%
                sum(na.rm = TRUE) %>%
                stats::setNames('MIXEDFOREST'))
    }
    if (any(grepl('SCRUB_MIXED', names(landscape_stack)))) {
      res = c(res,
              terra::subset(landscape_stack,
                            names(landscape_stack)[grepl('SCRUB_MIXED', names(landscape_stack))]) %>%
                sum(na.rm = TRUE) %>%
                stats::setNames('MIXEDSHRUB'))
    }
    if (any(grepl('SCRUB_SALIX', names(landscape_stack)))) {
      res = c(res,
              terra::subset(landscape_stack,
                            names(landscape_stack)[grepl('SCRUB_SALIX', names(landscape_stack))]) %>%
                sum(na.rm = TRUE) %>%
                stats::setNames('SALIXSHRUB'))
    }
    if (any(grepl('SCRUB_INTRO', names(landscape_stack)))) {
      res = c(res,
              terra::subset(landscape_stack,
                            names(landscape_stack)[grepl('SCRUB_INTRO', names(landscape_stack))]) %>%
                sum(na.rm = TRUE) %>%
                stats::setNames('INTROSCRUB'))
    }
    if (any(c('RICE', 'IDLE', 'URBAN', 'WATER') %in% names(landscape_stack))) {
      res = c(res,
              terra::subset(landscape_stack,
                            which(names(landscape_stack) %in%
                                    c('RICE', 'IDLE', 'URBAN', 'WATER'))))
    }

  } else if (SDM == 'waterbird_fall') {
    res = c(
      # combine all riparian, perennial crops, managed wetlands,
      # other wetlands, and woodland/scrub
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('RIPARIAN', names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('woodw'),
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('ORCHARD|VINEYARD',
                                     names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('orch'),
      terra::subset(
        landscape_stack, c('WETLAND_TIDAL', 'WETLAND_OTHER')) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('wet'),
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('WETLAND_MANAGED',
                                     names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('duwet'),
      terra::subset(
        landscape_stack, c('WOODLAND', 'SCRUB')) %>% sum(na.rm = TRUE) %>%
        stats::setNames('for'),
      # for fall, combine all grains
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('GRAIN', names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('grain'),
      # keep others as-is and rename to match model inputs
      terra::subset(landscape_stack,
                    c('PASTURE_ALFALFA', 'PASTURE_OTHER', 'GRASSLAND',
                      'IDLE', 'RICE', 'FIELD_CORN', 'ROW', 'FIELD_OTHER',
                      'WATER', 'URBAN', 'BARREN')) %>%
        stats::setNames(
          c('alf', 'ip', 'dryp', 'fal', 'rice', 'corn', 'row', 'field', 'water',
            'dev', 'barren'))
    )
  } else if (SDM == 'waterbird_win') {
    res = c(
      # combine all riparian, perennial crops, managed wetlands,
      # other wetlands, and woodland/scrub
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('RIPARIAN', names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('woodw'),
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('ORCHARD|VINEYARD',
                                     names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('orch'),
      terra::subset(
        landscape_stack, c('WETLAND_TIDAL', 'WETLAND_OTHER')) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('wet'),
      terra::subset(
        landscape_stack,
        names(landscape_stack)[grepl('WETLAND_MANAGED',
                                     names(landscape_stack))]) %>%
        sum(na.rm = TRUE) %>%
        stats::setNames('duwet'),
      terra::subset(
        landscape_stack, c('WOODLAND', 'SCRUB')) %>% sum(na.rm = TRUE) %>%
        stats::setNames('for'),
      # for winter, keep wheat separate from other grains
      terra::subset(
        landscape_stack, c('GRAIN&HAY_OTHER', 'GRAIN&HAY_WHEAT')) %>%
        stats::setNames(c('grain', 'ww')),
      # keep others as-is and rename to match model inputs
      terra::subset(
        landscape_stack,
        c('PASTURE_ALFALFA', 'PASTURE_OTHER', 'GRASSLAND',
          'IDLE', 'RICE', 'FIELD_CORN', 'ROW', 'FIELD_OTHER',
          'WATER', 'URBAN', 'BARREN')) %>%
        stats::setNames(
          c('alf', 'ip', 'dryp', 'fal', 'rice', 'corn', 'row', 'field', 'water',
            'dev', 'barren'))
    )
  }
  return(res)
}
